﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using System.IO;
using System.Threading.Tasks;
using System.Threading;

namespace IKstol
{
    public partial class FormSTM : Form
    {
        private bool isConnected = false;
        private SerialPort currentPort = null;
        private bool ComPortC = true;
        private bool EnableReadPort = true;
        private bool EnableReconnect = false;
        private char command_toAr = 'a';
        private char command_toArb = 'a';
        private int sendCurrentTemp = 0;
        private int sendCurrentTempb = 0;
        private long Ttop = 0;
        private long Tbottom = 0;
        private int corrTtop = 0;
        private int corrTbottom = 0;
        private bool ProfileTop = true;
        private bool LinearTop = true;
        private bool LinearBottom = true;
        private bool EnablePaint = true;

        //private int Temp[5];

        public FormSTM()
        {
            InitializeComponent();
            SetupGraphLabels();
            SetupPort();
        }

        private void buttonPbplus_Click(object sender, EventArgs e)
        {

        }

        private void buttonPbmin_Click(object sender, EventArgs e)
        {

        }
        public void SetupGraphLabels()
        {
            scottPlotUC1.fig.labelTitle = string.Format("Графики нагрева поверхностей");
            scottPlotUC1.fig.labelX = "Время (c)";
            scottPlotUC1.fig.labelY = "Температура (гр)";
            scottPlotUC1.fig.xAxis.min = 10;
            scottPlotUC1.fig.xAxis.max = 600;
            scottPlotUC1.fig.yAxis.min = 100;
            scottPlotUC1.fig.yAxis.max = 400;
            scottPlotUC1.fig.xAxis.pixelsPerTick = 40;
            scottPlotUC1.fig.yAxis.pixelsPerTick = 25;
            scottPlotUC1.fig.Zoom(.9, .6);
            scottPlotUC1.fig.AxisSet(0, 230, 0, 250);
            scottPlotUC1.SizeUpdate();

        }
        //--------------------------------------------------------------------------------------
        private void SetupPort() //private void arduinoButton_Click(object sender, EventArgs e)
        {
            comboBox1.Items.Clear();
            // Получаем список COM портов доступных в системе
            //string[] portnames = SerialPort.GetPortNames();
            string[] portnames = System.IO.Ports.SerialPort.GetPortNames();
            // Проверяем есть ли доступные
            if (portnames.Length == 0)
            {
                MessageBox.Show("COM PORT not found");
            }
            foreach (string portName in portnames)
            {
                //добавляем доступные COM порты в список           
                comboBox1.Items.Add(portName);
                Console.WriteLine(portnames.Length);
                if (portnames[0] != null)
                {
                    comboBox1.SelectedItem = portnames[0];
                }
            }
        }
        private void connectToSTM()
        {
            isConnected = true;
            string selectedPort = comboBox1.GetItemText(comboBox1.SelectedItem);
            //    serialPort1.PortName = selectedPort;
            //serialPort1.Open();
            // проверка на повторное открытие
            if (currentPort != null) return;
            // Создаем "новый" порт, здесь нужно указать "свой COM порт" к которому подключена Arduino
            currentPort = new SerialPort(selectedPort, 256000);
            connectButton.Text = "Disconnect";
            currentPort.DataReceived += new SerialDataReceivedEventHandler(OnDataReceived);
            // запускаем поток обращения к порту
            Task.Run(() => CicleReadPort());
            EnableReconnect = false;
        }

        private void disconnectFromSTM()
        {
            //while (!EnableReadPort)  
            if (currentPort.IsOpen)
            {
                isConnected = false;
                currentPort.Close();
                currentPort.Dispose();
                currentPort = null;
                connectButton.Text = "Connect";
            }
        }
        private void connectButton_Click(object sender, EventArgs e)
        {
            if (!isConnected)
            {
                connectToSTM();
            }
            else
            {
                disconnectFromSTM();
            }
        }

        private void Reconnectbutton_Click(object sender, EventArgs e)
        {
            if (!EnableReconnect) return;
            if (currentPort == null)
            {
                string selectedPort = comboBox1.GetItemText(comboBox1.SelectedItem);
                currentPort = new SerialPort(selectedPort, 256000);  //currentPort = new SerialPort(PorttextBox.Text, 14400);
                currentPort.DataReceived += new SerialDataReceivedEventHandler(OnDataReceived);
            }
            ComPortC = true;
            EnablePaint = true;
            Task.Run(() => CicleReadPort());
        }

        private void comportBut_Click(object sender, EventArgs e)
        {
            comboBox1.Items.Clear();
            // Получаем список COM портов доступных в системе
            //string[] portnames = SerialPort.GetPortNames();
            string[] portnames = System.IO.Ports.SerialPort.GetPortNames();
            // Проверяем есть ли доступные
            if (portnames.Length == 0)
            {
                MessageBox.Show("COM PORT not found");
            }
            foreach (string portName in portnames)
            {
                //добавляем доступные COM порты в список           
                comboBox1.Items.Add(portName);
                Console.WriteLine(portnames.Length);
                if (portnames[0] != null)
                {
                    comboBox1.SelectedItem = portnames[0];
                }
            }
        }
//=====================================================================================================================
        private void CicleReadPort()
        {
            try
            {
                if (currentPort == null) return;
                // пробуем открыть соединение с портом
                currentPort.Open();

                if (!currentPort.IsOpen)
                {
                    // если не удалось открыть порт удаляем его
                    currentPort.Dispose();
                    currentPort = null;
                    return;
                }
            }
            catch
            {
                try
                {
                    if (!currentPort.IsOpen) return;
                }
                catch { return; }
            }
     //       Task.Run(() => PaintTick());
            // Основной цикл записи данных в порт
            try
            {
                DateTime sca = DateTime.Now;
                while (ComPortC)
                {
                    // ожидаем завершение обработки чтения данных
                    sca = DateTime.Now;
                    while (!EnableReadPort)
                    {
                        Application.DoEvents();
                        Thread.Sleep(5);
                        if (!ComPortC) return;
                        // при первом включении
                        if ((DateTime.Now - sca) > TimeSpan.FromSeconds(10))
                        {
                            EnableReadPort = true;
                        }
                    }
                    // запись данных в STM                    
                    // изменение температуры верха
                    if (command_toAr == 'N')
                    {
                        currentPort.Write(command_toAr.ToString());
                        currentPort.Write(command_toArb.ToString());
                       currentPort.Write(sendCurrentTemp.ToString("000"));
                    }
                    else
                    // включить PID верха и греть до указанной температуры
                    if (command_toAr == 'A')
                    {
                        currentPort.Write(command_toAr.ToString());
                        currentPort.Write(command_toArb.ToString());
                        currentPort.Write(sendCurrentTemp.ToString("000"));
                    }
                    // остальные команды
                    else
                    {
                        currentPort.Write(command_toAr.ToString());
                        currentPort.Write(command_toArb.ToString());
                        // включить PID низа и греть до указанной температуры
                        if (command_toArb == 'G')
                        {
                            currentPort.Write(sendCurrentTempb.ToString("000"));
                        }
                        else
                        // изменение температуры низа
                        if (command_toArb == 'M')
                        {
                            currentPort.Write(sendCurrentTempb.ToString("000"));
                        }
                    }
                    // дефолтная комманда для работы обмена данными
                    command_toAr = 'a';
                    command_toArb = 'a';
                    EnableReadPort = false;
                    Connectlabel.BeginInvoke(new Action(() => //Connectlabel.Invoke(new Action(() =>
                    {
                        Connectlabel.BackColor = Color.Red; //Connectlabel.BackColor = Color.DarkRed;
                    }));
                }
                // закрываем порт
                if (currentPort.IsOpen)
                {
                    currentPort.Close();
                    currentPort.Dispose();
                    currentPort = null;
                }
                currentPort.DataReceived -= OnDataReceived;
            }
            catch { EnableReadPort = true; }
        }
        //--------------------------------------------------------------------------------------
        private void OnDataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // процедура обработки полученных данных
            try
            {
                EnableReadPort = false;
                bool enUp = false;
                List<string> ln = new List<string>();
                int count = currentPort.BytesToRead;
                long res = 0;
                // задержка
                for (int i = 0; i < 12; i++)
                {
                    Application.DoEvents();
                    Thread.Sleep(10);
                }
                Connectlabel.BeginInvoke(new Action(() =>  // Connectlabel.Invoke(new Action(() =>
                {
                    Connectlabel.BackColor = Color.DarkBlue;
                }));

                // если есть что читать в буфере
                if (count > 5)
                {
                    enUp = true;
                    String mv = currentPort.ReadExisting().Trim('\r');
                    // массив полученных значений
                    string[] tmp = mv.Split('\n');
                    // t верха
                    //if (long.TryParse(tmp[0], out res))
                    //{
                    //    res = res + (long)corrTtop;
                    //    Ttop = res;
                        //if (!PaintStopcheckBox.Checked && (DateTime.Now - UpdGr) > TimeSpan.FromMilliseconds(950))
                        //{
                        //    try
                        //    {
                        //        lock (Ttemp)
                        //        {
                        //            Ttemp.Add(new TopTemp((double)Ttop, (double)CurrPos));
                        //        }
                        //    }
                        //    catch { }
                        //    UpdGr = DateTime.Now;
                        //}
                        //TopNlabel.Invoke(new Action(() =>
                        //{
                        //    TopNlabel.Text = res.ToString("000");
                        //}));
                    //}
                    if (long.TryParse(tmp[1], out res))
                    {
                        TopClabel.BeginInvoke(new Action(() =>  //TopClabel.Invoke(new Action(() =>
                        {
                            TopClabel.Text = res.ToString("000");
                        }));
                    }

                    // t низа
                    try
                    {
                        //if (long.TryParse(tmp[4], out res))
                        //{
                        //    res = res + (long)corrTbottom;
                        //    Tbottom = res;
                        //    if (!PaintStopcheckBox.Checked && (DateTime.Now - UpdGrB) > TimeSpan.FromMilliseconds(950))
                        //    {
                        //        try
                        //        {
                        //            lock (Btemp)
                        //            {
                        //                Btemp.Add(new BottomTemp((double)Tbottom, (double)CurrPos));
                        //            }
                        //        }
                        //        catch { }
                        //        UpdGrB = DateTime.Now;
                        //    }
                        //}
                        //TopNblabel.Invoke(new Action(() =>
                        //{
                        //    TopNblabel.Text = res.ToString("000");
                        //}));
                        if (long.TryParse(tmp[5], out res))
                        {
                            TopCblabel.BeginInvoke(new Action(() =>   //TopCblabel.Invoke(new Action(() =>
                            {
                                TopCblabel.Text = res.ToString("000");
                            }));
                        }

                    }
                    catch { }
                    res = 1;
                    // запоминаем полученные значения, для последующего оттображения
                    foreach (var item in tmp)
                    {
                        ln.Add(item);
                    }
                }
                // выводим в TextBox что получили
                if (enUp && count > 5)
                {
                    textBoxOut.Invoke(new Action(() =>
                    {
                        textBoxOut.Lines = ln.ToArray();
                    }));
                }
                else
                {
                    textBoxOut.Invoke(new Action(() =>
                    {
                        textBoxOut.Text = "";
                    }));
                }
                EnableReadPort = true;
            }
            catch (Exception ex)
            { }
        }
        //--------------------------------------------------------------------------------------
        private void FormSTM_FormClosing(object sender, FormClosingEventArgs e)
        {
            EnablePaint = false;
            ProfileTop = false;
            LinearTop = false;
            LinearBottom = false;
            for (int i = 0; i < 10; i++)
            {
                Application.DoEvents();
                Thread.Sleep(1);
            }
            command_toAr = 'f';
            for (int i = 0; i < 30; i++)
            {
                Application.DoEvents();
                Thread.Sleep(1);
            }
            command_toArb = 'g';
            for (int i = 0; i < 30; i++)
            {
                Application.DoEvents();
                Thread.Sleep(1);
            }

            ComPortC = false;
            for (int i = 0; i < 10; i++)
            {
                Application.DoEvents();
                Thread.Sleep(1);
            }
            try
            {
                if (currentPort.IsOpen)
                {
                    currentPort.Close();
                    currentPort.Dispose();
                    currentPort = null;
                }
            }
            catch { }
        }
        //--------------------------------------------------------------------------------------
//=====================================================================================================================
        private void TopOnbutton_Click(object sender, EventArgs e)
        {
            ProfileTop = false;
            LinearTop = false;
            command_toAr = 'A'; //command_toAr = 't';
        }
        //--------------------------------------------------------------------------------------
        private void TopOffbutton_Click(object sender, EventArgs e)
        {
            ProfileTop = false;
            LinearTop = false;
            command_toAr = 'N'; //command_toAr = 'T';
        }
        //--------------------------------------------------------------------------------------
        private void BottomOnbutton_Click(object sender, EventArgs e)
        {
            command_toArb = 'b';
        }
        //--------------------------------------------------------------------------------------
        private void BottomOffbutton_Click(object sender, EventArgs e)
        {
            command_toArb = 'B';
        }
        //--------------------------------------------------------------------------------------

    }
}
